﻿using DataSharing.Province.SendTask.ProvinceOther;
using DataSharing.RawData;
using DataSharing.SendTask;
using DataSharing.Share.Dtos.Province;
using DataSharing.Share.Enums;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Quartz;
using System.Net;
using XF.Domain.Repository;

namespace DataSharing.Province.SendTask
{
    /// <summary>
    /// 
    /// </summary>
    public class TaskProvinceOtherJob : IJob, IDisposable
    {
        private readonly ILogger<TaskProvinceOtherJob> _logger;
        private readonly IRepository<DsSendTaskProvinceOther> _sendTaskProvinceOtherRepository;
        private readonly IRepository<DsWaitSendTaskProvinceOther> _waitSendTaskProvinceOtherRepository;
        private readonly IChannelConfigurationManager _channelConfigurationManager;
        private readonly HuiJuClient _huiJuClient;
        private readonly XieTongClient _xieTongClient;
        private readonly IRepository<DsSendTaskDetailInfo> _sendTaskDetailInfoRepository;
        private readonly IHttpClientFactory _httpClientFactory;
        private readonly IRepository<DsOrderVisitSend> _dsOrderVisitSendRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="sendTaskProvinceOtherRepository"></param>
        /// <param name="waitSendTaskProvinceOtherRepository"></param>
        /// <param name="channelConfigurationManager"></param>
        /// <param name="huiJuClient"></param>
        /// <param name="xieTongClient"></param>
        /// <param name="sendTaskDetailInfoRepository"></param>
        /// <param name="httpClientFactory"></param>
        /// <param name="dsOrderVisitSendRepository"></param>
        public TaskProvinceOtherJob(ILogger<TaskProvinceOtherJob> logger,
            IRepository<DsSendTaskProvinceOther> sendTaskProvinceOtherRepository,
            IRepository<DsWaitSendTaskProvinceOther> waitSendTaskProvinceOtherRepository,
            IChannelConfigurationManager channelConfigurationManager,
            HuiJuClient huiJuClient,
            XieTongClient xieTongClient,
           IRepository<DsSendTaskDetailInfo> sendTaskDetailInfoRepository,
           IHttpClientFactory httpClientFactory,
           IRepository<DsOrderVisitSend> dsOrderVisitSendRepository)
        {
            _logger = logger;
            _sendTaskProvinceOtherRepository = sendTaskProvinceOtherRepository;
            _waitSendTaskProvinceOtherRepository = waitSendTaskProvinceOtherRepository;
            _channelConfigurationManager = channelConfigurationManager;
            _huiJuClient = huiJuClient;
            _xieTongClient = xieTongClient;
            _sendTaskDetailInfoRepository = sendTaskDetailInfoRepository;
            _httpClientFactory = httpClientFactory;
            _dsOrderVisitSendRepository = dsOrderVisitSendRepository;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        public async Task Execute(IJobExecutionContext context)
        {
            var configProvince = _channelConfigurationManager.GetConfigurationProvince();
            //加载数据
            var tasks = await _waitSendTaskProvinceOtherRepository.Queryable()
           .Where(d => d.State == EWaitSendTaskState.WaitPush && d.SendTimes <= 6)
           .OrderBy(d => d.CreationTime)
           .Take(50)
           .ToListAsync(context.CancellationToken);

            if (tasks.Count != 0)
            {
                //处理推送数据
                foreach (var sendTask in tasks)
                {
                    //状态更改为推送中
                    sendTask.State = EWaitSendTaskState.Pushing;
                    //使用乐观锁，控制数据
                    if (await _waitSendTaskProvinceOtherRepository.Updateable(sendTask).ExecuteCommandWithOptLockAsync() > 0)
                    {
                        var name = GetName(sendTask.Path);

                        _logger.LogWarning("省请求参数：" + "--------------" + sendTask.Request);
                        ProvinceResponse result = new();
                        string error = "";
                        if (sendTask.PathType == EPathType.HuiJu) //汇聚推送
                        {
                            try
                            {
                                result = await _huiJuClient.RequestAsync<ProvinceResponse>(configProvince.HuiJu + sendTask.Path, sendTask.HttpMethod, sendTask.Request, context.CancellationToken);
                            }
                            catch (Exception ex)
                            {
                                error = ex.Message;
                            }
                        }
                        else if (sendTask.PathType == EPathType.XieTong)//协同推送
                        {
                            try
                            {
                                result = await _xieTongClient.RequestAsync<ProvinceResponse>(configProvince.XieTong + sendTask.Path, sendTask.HttpMethod, sendTask.Request, context.CancellationToken);
                            }
                            catch (Exception ex)
                            {
                                error = ex.Message;
                            }
                        }
                        else if (sendTask.PathType == EPathType.File)  //附件
                        {
                            if (sendTask != null && sendTask.FileJson != null && sendTask.FileJson.Count > 0)
                            {
                                var itemFile = sendTask.FileJson[0];
                                result = await SendFileData(sendTask.Request, itemFile.FileId, itemFile.FileId, context.CancellationToken);
                            }
                        }
                        _logger.LogWarning("省接口返回：" + name + "--------------" + System.Text.Json.JsonSerializer.Serialize(result));

                        //写入明细表
                        DsSendTaskDetailInfo dsSendTaskInfo = new()
                        {
                            TaskId = sendTask.Id,
                            Result = System.Text.RegularExpressions.Regex.Unescape(System.Text.Json.JsonSerializer.Serialize(result)),
                            ResultErrorData = error,
                            ProcessingServices = _channelConfigurationManager.GetConfigurationProcessingServices()
                        };
                        if (result is not null)
                        {
                            #region 处理待同步表数据
                            //移除待同步表数据
                            if (result.ReturnInfo is not null && result.ReturnInfo.Code == "1")
                                await _waitSendTaskProvinceOtherRepository.RemoveAsync(sendTask, cancellationToken: context.CancellationToken);
                            else
                            {
                                //修改待同步表数据状态
                                sendTask.LastTime = DateTime.Now;
                                sendTask.SendTimes = sendTask.SendTimes + 1;

                                if (sendTask.FirstTime is null)
                                    sendTask.FirstTime = DateTime.Now;

                                if (sendTask.SendTimes >= 7)
                                    sendTask.State = EWaitSendTaskState.PushFail;
                                else
                                    sendTask.State = EWaitSendTaskState.WaitPush;

                                await _waitSendTaskProvinceOtherRepository.UpdateAsync(sendTask, context.CancellationToken);
                            }
                            #endregion

                            #region 处理推送任务表
                            var sendTaskData = await _sendTaskProvinceOtherRepository.GetAsync(p => p.Id == sendTask.Id, context.CancellationToken);
                            if (sendTaskData != null)
                            {
                                //处理推送时间
                                sendTaskData.LastTime = DateTime.Now;
                                sendTaskData.SendTimes = sendTaskData.SendTimes + 1;

                                if (sendTaskData.FirstTime is null)
                                    sendTaskData.FirstTime = DateTime.Now;

                                if (result.ReturnInfo is not null && result.ReturnInfo.Code == "1")
                                {
                                    sendTaskData.IsSuccess = ESendTaskState.PushSuccess;
                                    dsSendTaskInfo.IsSuccess = true;
                                }
                                else
                                    sendTaskData.IsSuccess = ESendTaskState.PushFail;

                                await _sendTaskProvinceOtherRepository.UpdateAsync(sendTaskData, context.CancellationToken);

                            }
                            #endregion

                            #region 处理回访数据推送时间
                            if (result.ReturnInfo is not null && result.ReturnInfo.Code == "1")
                            {
                                if (sendTask.TaskType == "SubmitVisitInfo" || sendTask.TaskType == "GetVisitInfoReceive")
                                {
                                    var visitData = await _dsOrderVisitSendRepository.GetAsync(p => p.ProvinceNo == sendTaskData.ProvinceNo, context.CancellationToken);
                                    if (visitData != null)
                                    {
                                        if (visitData.FirstSendProvinceTime is null)
                                            visitData.FirstSendProvinceTime = DateTime.Now;

                                        visitData.LastSendProvinceTime = DateTime.Now;
                                        await _dsOrderVisitSendRepository.UpdateAsync(visitData, context.CancellationToken);
                                    }
                                }
                            }
                            #endregion
                        }
                        await _sendTaskDetailInfoRepository.AddAsync(dsSendTaskInfo, context.CancellationToken);
                    }
                }
            }
        }

        /// <summary>
        /// 附件读取以及附件上传
        /// </summary>
        /// <param name="request"></param>
        /// <param name="fileName"></param>
        /// <param name="fileId"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        private async Task<ProvinceResponse> SendFileData(string request, string fileName, string fileId, CancellationToken cancellationToken)
        {
            //获取配置信息
            var configProvince = _channelConfigurationManager.GetConfigurationProvince();
            var businessFile = _channelConfigurationManager.GetConfigurationBusinessFile();
            byte[] fileContentBytes = null;
            try
            {
                var baseAddress = businessFile.BaseUrl;
                if (!baseAddress.EndsWith('/'))
                    baseAddress += "/";
                string url = string.Format("{0}{1}?Source={2}&Id={3}", baseAddress, businessFile.DownloadUrlAddress, "hotline", fileId);
                using (var client = _httpClientFactory.CreateClient())
                using (var response = await client.GetAsync(url))
                {

                    if (response.StatusCode == HttpStatusCode.OK)
                    {
                        fileName = response.Content.Headers.ContentDisposition.FileNameStar;
                        using var memoryStream = new MemoryStream();
                        await response.Content.CopyToAsync(memoryStream);
                        memoryStream.Seek(0, SeekOrigin.Begin);

                        fileContentBytes = memoryStream.ToArray();
                    }
                }
                if (fileContentBytes != null)
                {
                    //获取附件上传Token
                    string strToken = await _xieTongClient.GetTokenAsync(cancellationToken);
                    Dictionary<string, object> dicParam = new()
                    {
                        { "params",  request } // 第一个接口参数，json格式字符串
                    };

                    // 构造字典文件数据
                    // 接口参数名称为files
                    CFormUpload.FileParameter fileParameter = new CFormUpload.FileParameter("files", fileContentBytes, fileName, null);
                    dicParam.Add(fileName, fileParameter);

                    var baseAddressHuiJu = configProvince.HuiJu;
                    if (!baseAddressHuiJu.EndsWith('/'))
                        baseAddressHuiJu += "/";

                    string strUrl = baseAddressHuiJu + "get_case_material_info";
                    // 上传附件
                    string strResult = CFormUpload.MultipartFormDataPost(strUrl, null, dicParam, strToken);

                    _logger.LogWarning("省接口上传附件返回：--------------" + strResult);

                    if (!string.IsNullOrEmpty(strResult))
                        return JsonConvert.DeserializeObject<ProvinceResponse>(strResult);
                }
                return new ProvinceResponse();
            }
            catch (Exception ex)
            {
                return new ProvinceResponse();
            }
        }

        /// <summary>
        /// 获取接口名称
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        private string GetName(string path)
        {
            string name = "";
            switch (path)
            {
                case "get_case_back_apply":
                    name = "业务协同-退回申请接口";
                    break;
                case "send_supervise_process_info":
                    name = "业务协同-服务工单督办过程接口";
                    break;
                case "send_supervise_result_info":
                    name = "业务协同-服务工单督办结果接口";
                    break;
                case "screen_case_info_send":
                    name = "业务协同-工单发起甄别接口";
                    break;
                case "delay_case_info_send":
                    name = "业务协同-服务工单申请延时接口";
                    break;
                case "submit_case_info":
                    name = "业务汇聚-服务工单受理接口";
                    break;
                case "submit_case_record":
                    name = "业务汇聚-电话记录";
                    break;
                case "get_case_process_receive":
                    name = "业务协同-服务工单交办处理过程接口";
                    break;
                case "submit_case_process":
                    name = "业务汇聚-服务工单处理过程接口";
                    break;
                case "submit_case_total":
                    name = "业务汇聚-统计数据接口";
                    break;
                case "supply_case_info_send":
                    name = "业务协同-上报补充诉求接口";
                    break;
                case "revoke_case_info":
                    name = "业务协同-服务工单撤单接口";
                    break;
                case "remind_case_info":
                    name = "业务协同-服务工单催单接口";
                    break;
                case "get_case_result_receive":
                    name = "业务协同-服务工单交办处理结果接口";
                    break;
                case "zmhd_case_info_public":
                    name = "业务协同-政民互动提交公开";
                    break;
                case "get_knowledge_info_send":
                    name = "业务协同-知识库上报接口";
                    break;
                case "get_knowledge_info_update":
                    name = "业务协同-知识库变更接口";
                    break;
                case "get_knowledge_info_abandon":
                    name = "业务协同-知识库下架接口";
                    break;
                case "sendrealtimestatus":
                    name = "实时服务上报接口";
                    break;
                case "get_visit_info_receive":
                    name = "业务协同-服务工单交办评价接口";
                    break;
                case "submit_visit_info":
                    name = "业务汇聚-服务工单回访评价接口";
                    break;
                case "submit_case_result":
                    name = "业务汇聚-服务工单处理结果接口";
                    break;
                case "submit_case_extends":
                    name = "业务汇聚-服务工单拓展信息接口";
                    break;
                case "send_case_info":
                    name = "业务协同-服务工单上报";
                    break;
                case "get_case_material_info":
                    name = "业务汇聚-材料接口";
                    break;
                default:
                    break;
            }

            return name;
        }

        public void Dispose()
        {

        }
    }
}