﻿using DataSharing.Province.SendTask.SubmitCaseProcess;
using DataSharing.SendTask;
using DataSharing.Share.Dtos.Province;
using DataSharing.Share.Enums;
using MapsterMapper;
using Microsoft.Extensions.Logging;
using Quartz;
using System.Text.Json;
using XF.Domain.Repository;

namespace DataSharing.Province.SendTask
{
    /// <summary>
    /// 服务工单处理过程  
    /// </summary>
    public class TaskSubmitCaseProcessJob : IJob, IDisposable
    {
        private readonly IMapper _mapper;
        private readonly ILogger<TaskSubmitCaseProcessJob> _logger;
        private readonly IRepository<DsSendTaskSubmitCaseProcess> _sendTaskSubmitCaseProcessRepository;
        private readonly IRepository<DsWaitSendTaskSubmitCaseProcess> _waitSendTaskSubmitCaseProcessRepository;
        private readonly IChannelConfigurationManager _channelConfigurationManager;
        private readonly HuiJuClient _huiJuClient;
        private readonly IRepository<DsSendTaskDetailInfo> _sendTaskDetailInfoRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="mapper"></param>
        /// <param name="logger"></param>
        /// <param name="sendTaskSubmitCaseProcessRepository"></param>
        /// <param name="waitSendTaskSubmitCaseProcessRepository"></param>
        /// <param name="channelConfigurationManager"></param>
        /// <param name="huiJuClient"></param>
        /// <param name="sendTaskDetailInfoRepository"></param>
        public TaskSubmitCaseProcessJob(IMapper mapper, ILogger<TaskSubmitCaseProcessJob> logger,
            IRepository<DsSendTaskSubmitCaseProcess> sendTaskSubmitCaseProcessRepository,
            IRepository<DsWaitSendTaskSubmitCaseProcess> waitSendTaskSubmitCaseProcessRepository,
            IChannelConfigurationManager channelConfigurationManager,
            HuiJuClient huiJuClient,
           IRepository<DsSendTaskDetailInfo> sendTaskDetailInfoRepository)
        {
            _mapper = mapper;
            _logger = logger;
            _sendTaskSubmitCaseProcessRepository = sendTaskSubmitCaseProcessRepository;
            _waitSendTaskSubmitCaseProcessRepository = waitSendTaskSubmitCaseProcessRepository;
            _channelConfigurationManager = channelConfigurationManager;
            _huiJuClient = huiJuClient;
            _sendTaskDetailInfoRepository = sendTaskDetailInfoRepository;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        public async Task Execute(IJobExecutionContext context)
        {
            var configProvince = _channelConfigurationManager.GetConfigurationProvince();
            //加载数据
            var tasks = await _waitSendTaskSubmitCaseProcessRepository.Queryable()
           .Where(d => d.State == EWaitSendTaskState.WaitPush && d.SendTimes <= 6)
           .OrderBy(d => d.CreationTime)
           .Take(50)
           .ToListAsync(context.CancellationToken);
            _logger.LogWarning("业务汇聚-服务工单处理过程读取数据条数--------------------------------------" + tasks.Count);
            if (tasks.Count != 0)
            {
                #region 处理请求参数
                var request = new ProvinceListRequest<Share.Dtos.Province.HuiJu.SubmitCaseProcessInfo>(configProvince.ClientId, configProvince.ClientSecret);
                List<Share.Dtos.Province.HuiJu.SubmitCaseProcessInfo> listOrder = [];
                #endregion

                List<DsWaitSendTaskSubmitCaseProcess> taskData = [];

                //处理推送数据
                foreach (var sendTask in tasks)
                {
                    //状态更改为推送中
                    sendTask.State = EWaitSendTaskState.Pushing;
                    //使用乐观锁，控制数据
                    if (await _waitSendTaskSubmitCaseProcessRepository.Updateable(sendTask).ExecuteCommandWithOptLockAsync() > 0)
                    {
                        //组装这一条数据
                        listOrder.Add(sendTask.RequestData);
                        taskData.Add(sendTask);
                    }
                }
                _logger.LogWarning("业务汇聚-服务工单处理过程锁定条数--------------------------------------" + taskData.Count);
                if (listOrder != null && listOrder.Count > 0)
                {
                    request.SetData(listOrder);
                    var jsonData = request.ToString();
                    _logger.LogWarning("业务汇聚-服务工单处理过程请求参数--------------------------------------" + jsonData);
                    ProvinceResponse result = new();
                    string error = "";
                    //执行推送任务
                    try
                    {
                        result = await _huiJuClient.RequestAsync<ProvinceResponse>(configProvince.HuiJu + PtahDefaults.SubmitCaseProcess, "Post", jsonData, context.CancellationToken);
                    }
                    catch (Exception ex)
                    {
                        error = ex.Message;
                    }
                    _logger.LogWarning("业务汇聚-服务工单处理过程返回数据-------------------------------------" + JsonSerializer.Serialize(result));
                    if (result != null)
                    {
                        //处理数据
                        foreach (var item in taskData)
                        {
                            //查询是否有错误
                            //有错误处理
                            //没有错误删除待推送表，更新主表
                            bool isError = false;
                            if (result == null || result.Custom == null || result.ReturnInfo == null)
                            {
                                //请求报错，那么则所有的推送失败
                                isError = true;
                            }
                            else
                            {
                                if (result.Custom.ErrorObjs.Exists(p => p.CaseSerial == item.ProvinceNo))
                                    isError = true;
                            }

                            //处理待推送数据
                            if (isError)
                            {
                                //修改
                                item.LastTime = DateTime.Now;
                                item.SendTimes = item.SendTimes + 1;

                                if (item.FirstTime is null)
                                    item.FirstTime = DateTime.Now;

                                if (item.SendTimes >= 7)
                                    item.State = EWaitSendTaskState.PushFail;
                                else
                                    item.State = EWaitSendTaskState.WaitPush;

                                //修改待推送表
                                await _waitSendTaskSubmitCaseProcessRepository.UpdateAsync(item, context.CancellationToken);
                            }
                            else
                            {
                                //删除
                                await _waitSendTaskSubmitCaseProcessRepository.RemoveAsync(item, cancellationToken: context.CancellationToken);
                            }

                            //查询主表
                            var sendData = await _sendTaskSubmitCaseProcessRepository.GetAsync(p => p.Id == item.Id, context.CancellationToken);
                            if (sendData != null)
                            {
                                sendData.LastTime = DateTime.Now;
                                sendData.SendTimes = sendData.SendTimes + 1;

                                if (sendData.FirstTime is null)
                                    sendData.FirstTime = DateTime.Now;

                                sendData.IsSuccess = isError == false ? ESendTaskState.PushSuccess : ESendTaskState.PushFail;
                                await _sendTaskSubmitCaseProcessRepository.UpdateAsync(sendData, cancellationToken: context.CancellationToken);
                            }

                            //新增明细
                            DsSendTaskDetailInfo dsSendTaskDetailInfo = new()
                            {
                                TaskId = item.Id,
                                IsSuccess = !isError,
                                Result = System.Text.RegularExpressions.Regex.Unescape(JsonSerializer.Serialize(result)),
                                ResultErrorData = error,
                                ProcessingServices = _channelConfigurationManager.GetConfigurationProcessingServices()
                            };
                            await _sendTaskDetailInfoRepository.AddAsync(dsSendTaskDetailInfo, context.CancellationToken);

                        }
                    }
                }
            }
        }



        public void Dispose()
        {

        }
    }
}
